package models

import (
	"hongze/hongze_cygx/utils"
	"rdluck_tools/orm"
	"time"
)

type CygxArticleHistoryRecord struct {
	Id          int `orm:"column(id);pk"`
	ArticleId   int
	UserId      int
	CreateTime  time.Time
	Mobile      string    `description:"手机号"`
	Email       string    `description:"邮箱"`
	CompanyId   int       `description:"公司id"`
	CompanyName string    `description:"公司名称"`
	ModifyTime  time.Time `description:"修改时间"`
}

//添加历史信息
func AddCygxArticleHistoryRecord(item *CygxArticleHistoryRecord) (lastId int64, err error) {
	o := orm.NewOrm()
	o.Begin()
	defer func() {
		if err == nil {
			o.Commit()
		} else {
			o.Rollback()
		}
	}()
	//var count int
	//sql := `SELECT COUNT(1) AS count FROM cygx_article_history_record WHERE user_id=? AND article_id=? `
	//err = o.Raw(sql, item.UserId, item.ArticleId).QueryRow(&count)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	//if count > 0 {
	//	sql := `UPDATE cygx_article_history_record SET modify_time=NOW() WHERE user_id=? AND article_id=? `
	//	_, err = o.Raw(sql, item.UserId, item.ArticleId).Exec()
	//} else {
	//	item.ModifyTime = time.Now()
	//	lastId, err = o.Insert(item)
	//}
	item.ModifyTime = time.Now()
	lastId, err = o.Insert(item)
	return
}

//获取用户阅读记录
func GetUserToArticleCount(uid, articleId int) (count int, err error) {
	sqlCount := `SELECT COUNT(1) AS count FROM cygx_article_history_record WHERE user_id=? AND article_id=? `
	o := orm.NewOrm()
	err = o.Raw(sqlCount, uid, articleId).QueryRow(&count)
	return
}

type AddStopTimeRep struct {
	ArticleId int `description:"文章ID"`
	StopTime  int `description:"停留时间"`
}

type AddStopTimeNewRep struct {
	Id        int `description:"ID"`
	ArticleId int `description:"文章ID"`
	StopTime  int `description:"停留时间"`
}

type ArticleDetailAddStopTimeRep struct {
	HasPermission int `description:"1:有该行业权限,正常展示，2：无该行业权限，不存在权益客户下，3:无该品类权限,已提交过申请，4：无该行业权限,未提交过申请，5：潜在客户，未提交过申请，6：潜在客户,已提交过申请"`
	HasFree       int `description:"1:已付费(至少包含一个品类的权限)，2:未付费(没有任何品类权限)"`
}

func AddArticleStopTime(item *AddStopTimeNewRep) (err error) {
	o := orm.NewOrm()
	sql := `UPDATE cygx_article_history_record SET stop_time = ? WHERE id =?`
	_, err = o.Raw(sql, item.StopTime, item.Id).Exec()
	return
}

//获取最新的一条阅读记录
func GetNewArticleHistoryRecord(uid, articleId int) (item *AddStopTimeNewRep, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM  cygx_article_history_record WHERE user_id = ? AND article_id = ? ORDER BY id DESC LIMIT 1;`
	err = o.Raw(sql, uid, articleId).QueryRow(&item)
	return
}
